@@ -1,6 +1,3 @@
-/* $Id: color.h,v 1.8 2011/01/25 16:30:48 ellson Exp $ $Revision: 1.8 $ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,13 +5,10 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifndef GV_COLOR_H
-#define GV_COLOR_H
-
-/* #include "arith.h" */
+#pragma once
 
 #ifdef __cplusplus
 extern "C" {
@@ -51,4 +45,3 @@ typedef struct color_s {
 #ifdef __cplusplus
 }
 #endif
-#endif
