@@ -1,6 +1,3 @@
-/* $Id: gvcjob.h,v 1.84 2011/01/25 16:30:49 ellson Exp $ $Revision: 1.84 $ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,13 +5,12 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /* Common header used by both clients and plugins */
 
-#ifndef GVCJOB_H
-#define GVCJOB_H
+#pragma once
 
 #ifdef __cplusplus
 extern "C" {
@@ -22,6 +18,7 @@ extern "C" {
 
 #include "gvcommon.h"
 #include "color.h"
+#include <stdbool.h>
 
 #define ARRAY_SIZE(A) (sizeof(A)/sizeof(A[0]))
 
@@ -33,7 +30,7 @@ extern "C" {
     typedef struct gvloadimage_engine_s gvloadimage_engine_t;
 
     typedef enum { PEN_NONE, PEN_DASHED, PEN_DOTTED, PEN_SOLID } pen_type;
-    typedef enum { FILL_NONE, FILL_SOLID } fill_type;
+    typedef enum { FILL_NONE, FILL_SOLID, FILL_LINEAR, FILL_RADIAL } fill_type;
     typedef enum { FONT_REGULAR, FONT_BOLD, FONT_ITALIC } font_type;
     typedef enum { LABEL_PLAIN, LABEL_HTML } label_type;
 
@@ -57,7 +54,7 @@ extern "C" {
  GVDEVICE_DOES_TRUECOLOR	supports alpha channel -Tpng, -Tgtk, -Txlib 
  GVDEVICE_BINARY_FORMAT		Suppresses \r\n substitution for linends 
  GVDEVICE_COMPRESSED_FORMAT	controls libz compression		
- GVDEVICE_NO_WRITER		used when gvdevice is not used because device uses its own writer, -Tming, devil outputs   (FIXME seems to overlap OUTPUT_NOT_REQUIRED)
+ GVDEVICE_NO_WRITER		used when gvdevice is not used because device uses its own writer, devil outputs   (FIXME seems to overlap OUTPUT_NOT_REQUIRED)
 
  GVRENDER_Y_GOES_DOWN		device origin top left, y goes down, otherwise
   				device origin lower left, y goes up	
@@ -152,7 +149,7 @@ extern "C" {
 	int argc;
 	int alloc;
     } gv_argvlist_t;
-
+    
     typedef struct gvdevice_callbacks_s {
 	void (*refresh) (GVJ_t * job);
         void (*button_press) (GVJ_t * job, int button, pointf pointer);
@@ -200,7 +197,9 @@ extern "C" {
 
 	emit_state_t emit_state; 
 
-	gvcolor_t pencolor, fillcolor;
+	gvcolor_t pencolor, fillcolor, stopcolor;
+	int gradient_angle;
+	float gradient_frac;
 	pen_type pen;
 	fill_type fill;
 	double penwidth;
@@ -230,22 +229,23 @@ extern "C" {
 	char *tailtarget;
 	char *headtarget; 
 
-	int explicit_tooltip:1;
-	int explicit_tailtooltip:1;
-	int explicit_headtooltip:1;
-	int explicit_labeltooltip:1;
-	int explicit_tailtarget:1;
-	int explicit_headtarget:1;
-	int explicit_edgetarget:1;
-	int explicit_tailurl:1;
-	int explicit_headurl:1;
+	unsigned explicit_tooltip:1;
+	unsigned explicit_tailtooltip:1;
+	unsigned explicit_headtooltip:1;
+	unsigned explicit_labeltooltip:1;
+	unsigned explicit_tailtarget:1;
+	unsigned explicit_headtarget:1;
+	unsigned explicit_edgetarget:1;
+	unsigned explicit_tailurl:1;
+	unsigned explicit_headurl:1;
+	unsigned labeledgealigned:1;
 
 	/* primary mapped region - node shape, edge labels */
 	map_shape_t url_map_shape; 
 	int url_map_n;                  /* number of points for url map if GVRENDER_DOES_MAPS */
 	pointf *url_map_p;
 
-	/* additonal mapped regions for edges */
+	/* additional mapped regions for edges */
 	int url_bsplinemap_poly_n;      /* number of polygons in url bspline map
 					 if GVRENDER_DOES_MAPS && GVRENDER_DOES_MAP_BSPLINES */
 	int *url_bsplinemap_n;          /* array of url_bsplinemap_poly_n ints 
@@ -293,13 +293,13 @@ extern "C" {
 	gvplugin_active_loadimage_t loadimage;
 	gvdevice_callbacks_t *callbacks;
 	pointf device_dpi;
-	boolean device_sets_dpi;
+	bool device_sets_dpi;
 
 	void *display;
 	int screen;
 
 	void *context;		/* gd or cairo surface */
-	boolean external_context;	/* context belongs to caller */
+	bool external_context;	/* context belongs to caller */
 	char *imagedata;	/* location of imagedata */
 
         int flags;		/* emit_graph flags */
@@ -339,7 +339,7 @@ extern "C" {
 	pointf  translation;    /* composite translation */
 	pointf  devscale;	/* composite device to points: dpi, y_goes_down */
 
-	boolean	fit_mode,
+	bool	fit_mode,
 		needs_refresh,
 		click,
 		has_grown,
@@ -371,4 +371,3 @@ extern "C" {
 #ifdef __cplusplus
 }
 #endif
-#endif				/* GVCJOB_H */
